@@ -66,7 +66,8 @@ static struct ipt_log_names ipt_log_names[]
 static u_int8_t
 parse_level(const char *level)
 {
-	unsigned int lev;
+	unsigned int lev = -1;
+	unsigned int set = 0;
 
 	if (string_to_number(level, 0, 7, &lev) == -1) {
 		unsigned int i = 0;
@@ -76,7 +77,7 @@ parse_level(const char *level)
 		     i++) {
 			if (strncasecmp(level, ipt_log_names[i].name,
 					strlen(level)) == 0) {
-				if (lev != -1)
+				if (set++)
 					exit_error(PARAMETER_PROBLEM,
 						   "log-level `%s' ambiguous",
 						   level);
@@ -84,7 +85,7 @@ parse_level(const char *level)
 			}
 		}
 
-		if (lev == -1)
+		if (!set)
 			exit_error(PARAMETER_PROBLEM,
 				   "log-level `%s' unknown", level);
 	}
@@ -113,7 +114,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 			exit_error(PARAMETER_PROBLEM,
 				   "Can't specify --log-level twice");
 
-		if (check_inverse(optarg, &invert))
+		if (check_inverse(optarg, &invert, NULL, 0))
 			exit_error(PARAMETER_PROBLEM,
 				   "Unexpected `!' after --log-level");
 
@@ -126,7 +127,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 			exit_error(PARAMETER_PROBLEM,
 				   "Can't specify --log-prefix twice");
 
-		if (check_inverse(optarg, &invert))
+		if (check_inverse(optarg, &invert, NULL, 0))
 			exit_error(PARAMETER_PROBLEM,
 				   "Unexpected `!' after --log-prefix");
 
