@@ -22,10 +22,10 @@ static const struct reject_names reject_table[] = {
 		IPT_ICMP_NET_UNREACHABLE, "ICMP network unreachable"},
 	{"icmp-host-unreachable", "host-unreach",
 		IPT_ICMP_HOST_UNREACHABLE, "ICMP host unreachable"},
-	{"icmp-port-unreachable", "port-unreach",
-		IPT_ICMP_PORT_UNREACHABLE, "ICMP port unreachable (default)"},
 	{"icmp-proto-unreachable", "proto-unreach",
 		IPT_ICMP_PROT_UNREACHABLE, "ICMP protocol unreachable"},
+	{"icmp-port-unreachable", "port-unreach",
+		IPT_ICMP_PORT_UNREACHABLE, "ICMP port unreachable (default)"},
 #if 0
 	{"echo-reply", "echoreply",
 	 IPT_ICMP_ECHOREPLY, "for ICMP echo only: faked ICMP echo reply"},
@@ -97,7 +97,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 
 	switch(c) {
 	case '1':
-		if (check_inverse(optarg, &invert))
+		if (check_inverse(optarg, &invert, NULL, 0))
 			exit_error(PARAMETER_PROBLEM,
 				   "Unexpected `!' after --reject-with");
 		for (i = 0; i < limit; i++) {
@@ -115,6 +115,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 		exit_error(PARAMETER_PROBLEM, "unknown reject type `%s'",optarg);
 	default:
 		/* Fall through */
+		break;
 	}
 	return 0;
 }
@@ -146,10 +147,16 @@ static void save(const struct ipt_ip *ip, const struct ipt_entry_target *target)
 {
 	const struct ipt_reject_info *reject
 		= (const struct ipt_reject_info *)target->data;
+	unsigned int i;
 
-	printf("--reject-with %s ", reject_table[reject->with].name);
+	for (i = 0; i < sizeof(reject_table)/sizeof(struct reject_names); i++)
+		if (reject_table[i].with == reject->with)
+			break;
+
+	printf("--reject-with %s ", reject_table[i].name);
 }
 
+static
 struct iptables_target reject
 = { NULL,
     "REJECT",
