@@ -2,9 +2,9 @@
  * (C) 2000 by Harald Welte <laforge@gnumonks.org>
  * based on previous code from Rusty Russell <rusty@linuxcare.com.au>
  *
- * This coude is distributed under the terms of GNU GPL
+ * This code is distributed under the terms of GNU GPL
  *
- * $Id: iptables-restore.c,v 1.14 2001/08/06 18:50:22 laforge Exp $
+ * $Id: iptables-restore.c,v 1.20 2002/01/17 20:43:10 laforge Exp $
  */
 
 #include <getopt.h>
@@ -27,7 +27,7 @@ static int binary = 0, counters = 0, verbose = 0, noflush = 0;
 static struct option options[] = {
 	{ "binary", 0, 0, 'b' },
 	{ "counters", 0, 0, 'c' },
-/*	{ "verbose", 1, 0, 'v' }, */
+	{ "verbose", 1, 0, 'v' },
 	{ "help", 0, 0, 'h' },
 	{ "noflush", 0, 0, 'n'},
 	{ "modprobe", 1, 0, 'M'},
@@ -81,6 +81,7 @@ static int newargc;
 /* function adding one argument to newargv, updating newargc 
  * returns true if argument added, false otherwise */
 static int add_argv(char *what) {
+	DEBUGP("add_argv: %s\n", what);
 	if (what && ((newargc + 1) < sizeof(newargv)/sizeof(char *))) {
 		newargv[newargc] = strdup(what);
 		newargc++;
@@ -121,6 +122,9 @@ int main(int argc, char *argv[])
 			case 'c':
 				counters = 1;
 				break;
+			case 'v':
+				verbose = 1;
+				break;
 			case 'h':
 				print_usage("iptables-restore",
 					    NETFILTER_VERSION);
@@ -335,6 +339,16 @@ int main(int argc, char *argv[])
 					strncpy(param_buffer, param_start,
 						param_len);
 					*(param_buffer+param_len) = '\0';
+
+					/* check if table name specified */
+					if (!strncmp(param_buffer, "-t", 3)
+                                            || !strncmp(param_buffer, "--table", 8)) {
+						exit_error(PARAMETER_PROBLEM, 
+						   "Line %u seems to have a "
+						   "-t table option.\n", line);
+						exit(1);
+					}
+
 					add_argv(param_buffer);
 					param_start += param_len + 1;
 				} else {
@@ -345,7 +359,7 @@ int main(int argc, char *argv[])
 			DEBUGP("calling do_command(%u, argv, &%s, handle):\n",
 				newargc, curtable);
 
-			for (a = 0; a <= newargc; a++)
+			for (a = 0; a < newargc; a++)
 				DEBUGP("argv[%u]: %s\n", a, newargv[a]);
 
 			ret = do_command(newargc, newargv, 
